%% 用于演示基于自适应降噪方法的代码,可以直接运行。
% 程序运行完成后，在MATLAB的工作区，双击filtered_signal4变量，可以查看滤波得到的具体数值。
% 本程序2023.05定型，后续更新都在完整版代码中
% 知乎原文链接：https://zhuanlan.zhihu.com/p/608044525/
% 完整版代码地址：http://khsci.com/docs/index.php/emdfilter/
% 注意！公开版代码需使用2022a及以上MATLAB版本，完整版代码可根据代码说明适用相对较低版本MATLAB
clear all
versionTest;% 代码运行环境监测
%% 1.生成仿真信号，x为无噪声信号，sig为添加噪声后的信号
rng(123); %设置随机种子，保证每次生成噪声的一致性
N = 200;
t = linspace(0,1,N);
x = 4*sin(4*pi*t);
x = x - sign(t-0.3) - sign(0.72-t);
sig = x + 0.2*randn(size(t));
figure('Color','w');subplot(2,1,1);plot(t,x);title('未加入白噪声信号')
subplot(2,1,2);plot(t,sig);title('加入白噪声信号')

%% 2. 对信号进行EMD分解
imfs = pEMDandFFT(sig,1);

%% 3.调用filAdaptive实现滤波
[filtered_signal4, RPi_values] = filAdaptive(imfs);
% filAdaptive 自适应降噪方法
% 参考论文：陈仁祥,汤宝平,马婧华.基于EEMD的振动信号自适应降噪方法[J].振动与冲击,2012,31(15):82-86.DOI:10.13465/j.cnki.jvs.2012.15.015.
%
% 输入:
% imfs - EMD分解得到的IMF分量，每一行对应一个IMF分量
%
% 输出:
% filtered_signal - 滤波后的信号，一维行向量
% RPi_values - 每个IMF分量的RPi值，一维行向量

%% 4.进行滤波指标评价
% 需要注意，只有在知道纯净信号（即未添加噪声信号）时才能进行指标评价
% 对于真实采样信号，无法获取其纯净信号值的，无法进行该步骤，需删除本小节代码
ori = x;  %无噪声信号
fil = filtered_signal4;  %滤波后信号
[SNR,MSE,NCC] = FilterEffectEvaluation(ori,fil);
% 对滤波效果进行评估
% 目前包含的评估指标有：信噪比SNR，均方差MSE，波形相似系数NCC
% 输入：
% ori：无噪声的原始数据，一维序列
% fil：滤波后的数据，一维序列
disp(['滤波后的SNR、MSE和NCC值分别为：',num2str(SNR),'、',num2str(MSE),'、',num2str(NCC)]);
%% 5.画滤波前后对比图
figure('color','w')
subplot(311);plot(x,'k');title('原始信号（未加入噪声）')
subplot(312);plot(sig,'k');title('原始信号（加入噪声）')
subplot(313);plot(fil,'k');title('滤波后信号')
% 注意，对于不知道纯净信号的情况，无法绘制“未加入噪声”的信号图像
% 此时需要将上述4行代码删除，并使用下述3行代码：
% figure('color','w')
% subplot(211);plot(sig,'k');title('原始信号')
% subplot(212);plot(s,'k');title('滤波后信号')

%% 关于完整版代码：
% 公开版代码的函数文件为p文件，可以被调用，但无法查看代码。
% 完整版代码中全部为m文件，m文件可以查看源码并自由修改
% 此外：公开版代码仅可对长度200以内的数据滤波；完整版无长度限制。

% 如果需要封装好的函数的完整版源码，可在下述链接获取：
% http://khsci.com/docs/index.php/emdfilter/
% 编程不易，感谢支持~

function versionTest
% 获取MATLAB版本
versionStr = version('-release');
versionYear = str2double(versionStr(1:4));

% 判断MATLAB版本是否为2022a或以上
if (versionYear < 2022)
% 如果版本过低，弹出提示框
warndlg('该代码需使用MATLAB 2022及更新版本运行', '版本警告');
end
end